@@ -11,6 +11,7 @@
 #include "FeaturedModel.h"
 #include "PaginateModel.h"
 #include "UnityLauncher.h"
+#include <Transaction/TransactionModel.h>
 
 // Qt includes
 #include "discover_debug.h"
@@ -38,7 +39,10 @@
 #include <KCrash>
 #include <KLocalizedContext>
 #include <KLocalizedString>
+#include <KOSRelease>
 #include <KSharedConfig>
+#include <KStatusNotifierItem>
+#include <KUiServerV2JobTracker>
 #include <kcoreaddons_version.h>
 // #include <KSwitchLanguageDialog>
 
@@ -59,6 +63,7 @@
 #ifdef WITH_FEEDBACK
 #include "plasmauserfeedback.h"
 #endif
+#include "PowerManagementInterface.h"
 #include "discoversettings.h"
 
 class CachedNetworkAccessManagerFactory : public QQmlNetworkAccessManagerFactory
@@ -100,6 +105,7 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
     qmlRegisterType<UnityLauncher>("org.kde.discover.app", 1, 0, "UnityLauncher");
     qmlRegisterType<PaginateModel>("org.kde.discover.app", 1, 0, "PaginateModel");
     qmlRegisterType<FeaturedModel>("org.kde.discover.app", 1, 0, "FeaturedModel");
+    qmlRegisterType<PowerManagementInterface>("org.kde.discover.app", 1, 0, "PowerManagementInterface");
     qmlRegisterType<OurSortFilterProxyModel>("org.kde.discover.app", 1, 0, "QSortFilterProxyModel");
 #ifdef WITH_FEEDBACK
     qmlRegisterSingletonType<PlasmaUserFeedback>("org.kde.discover.app", 1, 0, "UserFeedbackSettings", [](QQmlEngine *, QJSEngine *) -> QObject * {
@@ -145,11 +151,36 @@ DiscoverObject::DiscoverObject(CompactMode mode, const QVariantMap &initialPrope
             for (auto b : backends)
                 found |= b->hasApplications();
 
-            if (!found)
-                Q_EMIT openErrorPage(
-                    i18n("Discover currently cannot be used to install any apps "
-                         "because none of its app backends are available. Please "
-                         "report this issue to the packagers of your distribution."));
+            if (!found) {
+                QString errorText = i18n(
+                    "Discover currently cannot be used to install any apps or "
+                    "perform system updates because none of its app backends are "
+                    "available.");
+                QString errorExplanation = xi18nc("@info",
+                    "You can install some on the Settings page, under the "
+                    "<interface>Missing Backends</interface> section.<nl/><nl/>"
+                    "Also please consider reporting this as a packaging issue to "
+                    "your distribution.");
+                QString buttonIcon = QStringLiteral("tools-report-bug");
+                QString buttonText = i18n("Report This Issue");
+                QString buttonUrl = KOSRelease().bugReportUrl();
+
+                if (KOSRelease().name().contains(QStringLiteral("Arch Linux"))) {
+                    errorExplanation = xi18nc("@info",
+                        "You can use <command>pacman</command> to "
+                        "install the optional dependencies that are needed to "
+                        "enable the application backends.<nl/><nl/>Please note "
+                        "that Arch Linux developers recommend using "
+                        "<command>pacman</command> for managing software because "
+                        "the PackageKit backend is not well-integrated on Arch "
+                        "Linux.");
+                    buttonIcon = QStringLiteral("help-about");
+                    buttonText = i18n("Learn More");
+                    buttonUrl = KOSRelease().supportUrl();
+                }
+
+                Q_EMIT openErrorPage(errorText, errorExplanation, buttonText, buttonIcon, buttonUrl);
+            }
         },
         this);
 
@@ -303,11 +334,15 @@ void DiscoverObject::openApplication(const QUrl &url)
                     openApplication(QUrl(QStringLiteral("appstream://org.kde.discover.snap")));
                     showPassiveNotification(i18n("Please make sure Snap support is installed"));
                 } else {
-                    Q_EMIT openErrorPage(
-                        i18n("Could not open %1 because it was not found in any "
-                             "available software repositories. Please report this "
-                             "issue to the packagers of your distribution.",
-                             url.toDisplayString()));
+                    const QString errorText = i18n("Could not open %1 because it "
+                    "was not found in any available software repositories.",
+                    url.toDisplayString());
+                    const QString errorExplanation = i18n("Please report this "
+                    "issue to the packagers of your distribution.");
+                    QString buttonIcon = QStringLiteral("tools-report-bug");
+                    QString buttonText = i18n("Report This Issue");
+                    QString buttonUrl = KOSRelease().bugReportUrl();
+                    Q_EMIT openErrorPage(errorText, errorExplanation, buttonText, buttonIcon, buttonUrl);
                 }
             });
         },
@@ -320,6 +355,96 @@ void DiscoverObject::openApplication(const QUrl &url)
     }
 }
 
+class TransactionsJob : public KJob
+{
+public:
+    void start() override
+    {
+        // no-op, this is just observing
+
+        setTotalAmount(Items, TransactionModel::global()->rowCount());
+        setPercent(TransactionModel::global()->progress());
+        connect(TransactionModel::global(), &TransactionModel::lastTransactionFinished, this, &TransactionsJob::emitResult);
+        connect(TransactionModel::global(), &TransactionModel::transactionRemoved, this, &TransactionsJob::refreshInfo);
+        connect(TransactionModel::global(), &TransactionModel::progressChanged, this, [this] {
+            setPercent(TransactionModel::global()->progress());
+        });
+        refreshInfo();
+    }
+
+    void refreshInfo()
+    {
+        if (TransactionModel::global()->rowCount() == 0) {
+            return;
+        }
+
+        setProcessedAmount(Items, totalAmount(Items) - TransactionModel::global()->rowCount() + 1);
+        auto firstTransaction = TransactionModel::global()->transactions().constFirst();
+        Q_EMIT description(this, firstTransaction->name());
+    }
+
+    void cancel()
+    {
+        setError(KJob::KilledJobError /*KIO::ERR_USER_CANCELED*/);
+        deleteLater();
+    }
+};
+
+bool DiscoverObject::quitWhenIdle()
+{
+    if (!ResourcesModel::global()->isBusy()) {
+        return true;
+    }
+
+    if (!m_sni) {
+        auto tracker = new KUiServerV2JobTracker(m_sni);
+
+        m_sni = new KStatusNotifierItem(this);
+        m_sni->setStatus(KStatusNotifierItem::Active);
+        m_sni->setIconByName("plasmadiscover");
+        m_sni->setTitle(i18n("Discover"));
+        m_sni->setToolTip("process-working-symbolic", i18n("Discover"), i18n("Discover was closed before certain tasks were done, waiting for it to finish."));
+        m_sni->setStandardActionsEnabled(false);
+
+        connect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
+        connect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
+
+        auto job = new TransactionsJob;
+        job->setParent(this);
+        tracker->registerJob(job);
+        job->start();
+        connect(m_sni, &KStatusNotifierItem::activateRequested, job, &TransactionsJob::cancel);
+
+        rootObject()->hide();
+    }
+    return false;
+}
+
+void DiscoverObject::restore()
+{
+    if (!m_sni) {
+        return;
+    }
+
+    disconnect(TransactionModel::global(), &TransactionModel::countChanged, this, &DiscoverObject::reconsiderQuit);
+    disconnect(m_sni, &KStatusNotifierItem::activateRequested, this, &DiscoverObject::restore);
+
+    rootObject()->show();
+    m_sni->deleteLater();
+    m_sni = nullptr;
+}
+
+void DiscoverObject::reconsiderQuit()
+{
+    if (ResourcesModel::global()->isBusy()) {
+        return;
+    }
+
+    m_sni->deleteLater();
+    // Let the job UI to finalise properly
+    QTimer::singleShot(20, qGuiApp, &QCoreApplication::quit);
+}
+
 void DiscoverObject::integrateObject(QObject *object)
 {
     if (!object) {
@@ -349,8 +474,7 @@ void DiscoverObject::integrateObject(QObject *object)
 
     object->setParent(m_engine);
     connect(qGuiApp, &QGuiApplication::commitDataRequest, this, [this](QSessionManager &sessionManager) {
-        if (ResourcesModel::global()->isBusy()) {
-            Q_EMIT preventedClose();
+        if (!quitWhenIdle()) {
             sessionManager.cancel();
         }
     });
@@ -362,9 +486,7 @@ bool DiscoverObject::eventFilter(QObject *object, QEvent *event)
         return false;
 
     if (event->type() == QEvent::Close) {
-        if (ResourcesModel::global()->isBusy()) {
-            qCWarning(DISCOVER_LOG) << "not closing because there's still pending tasks";
-            Q_EMIT preventedClose();
+        if (!quitWhenIdle()) {
             return true;
         }
 
@@ -430,11 +552,6 @@ public:
 
     void finish()
     {
-        // The CI doesn't seem to have icons, remove when it's not an issue anymore
-        m_warnings.erase(std::remove_if(m_warnings.begin(), m_warnings.end(), [](const QQmlError &err) -> bool {
-            return err.description().contains(QLatin1String("QML Image: Failed to get image from provider: image://icon/"));
-        }));
-
         if (m_warnings.isEmpty())
             qCDebug(DISCOVER_LOG) << "cool no warnings!";
         else
