@@ -20,6 +20,13 @@ CategoryModel::CategoryModel(QObject *parent)
     t->setSingleShot(true);
     connect(t, &QTimer::timeout, this, &CategoryModel::populateCategories);
     connect(ResourcesModel::global(), &ResourcesModel::backendsChanged, t, QOverload<>::of(&QTimer::start));
+
+    // Use a timer so to compress the rootCategoriesChanged signal
+    // It generally triggers when KNS is unavailable at large (as explained in bug 454442)
+    m_rootCategoriesChanged = new QTimer(this);
+    m_rootCategoriesChanged->setInterval(0);
+    m_rootCategoriesChanged->setSingleShot(true);
+    connect(m_rootCategoriesChanged, &QTimer::timeout, this, &CategoryModel::rootCategoriesChanged);
 }
 
 CategoryModel *CategoryModel::global()
@@ -37,7 +44,7 @@ void CategoryModel::populateCategories()
 
     QVector<Category *> ret;
     CategoriesReader cr;
-    Q_FOREACH (const auto backend, backends) {
+    for (const auto backend : backends) {
         if (!backend->isValid())
             continue;
 
@@ -46,13 +53,13 @@ void CategoryModel::populateCategories()
         if (ret.isEmpty()) {
             ret = cats;
         } else {
-            Q_FOREACH (Category *c, cats)
+            for (Category *c : cats)
                 Category::addSubcategory(ret, c);
         }
     }
     if (m_rootCategories != ret) {
         m_rootCategories = ret;
-        Q_EMIT rootCategoriesChanged();
+        m_rootCategoriesChanged->start();
     }
 }
 
@@ -67,7 +74,7 @@ void CategoryModel::blacklistPlugin(const QString &name)
 {
     const bool ret = Category::blacklistPluginsInVector({name}, m_rootCategories);
     if (ret) {
-        Q_EMIT rootCategoriesChanged();
+        m_rootCategoriesChanged->start();
     }
 }
 
@@ -77,7 +84,7 @@ static Category *recFindCategory(Category *root, const QString &name)
         return root;
     else {
         const auto subs = root->subCategories();
-        Q_FOREACH (Category *c, subs) {
+        for (Category *c : subs) {
             Category *ret = recFindCategory(c, name);
             if (ret)
                 return ret;
