@@ -29,6 +29,7 @@ class FlatpakResource : public AbstractResource
     Q_PROPERTY(QStringList objects MEMBER s_bottomObjects CONSTANT)
     Q_PROPERTY(QString attentionText READ attentionText CONSTANT)
     Q_PROPERTY(QString dataLocation READ dataLocation CONSTANT)
+    Q_PROPERTY(QString branch READ branch CONSTANT)
     Q_PROPERTY(bool hasDataButUninstalled READ hasDataButUninstalled NOTIFY hasDataButUninstalledChanged)
 public:
     explicit FlatpakResource(const AppStream::Component &component, FlatpakInstallation *installation, FlatpakBackend *parent);
@@ -111,12 +112,14 @@ public:
     QUrl helpURL() override;
     QUrl bugURL() override;
     QUrl donationURL() override;
+    QUrl contributeURL() override;
     FlatpakFileType flatpakFileType() const;
     QString flatpakName() const;
     QJsonArray licenses() override;
     QString longDescription() override;
     QString name() const override;
     QString origin() const override;
+    QString displayOrigin() const override;
     QString packageName() const override;
     PropertyState propertyState(PropertyKind kind) const;
     QUrl resourceFile() const;
@@ -153,6 +156,7 @@ public:
     void setFlatpakFileType(FlatpakFileType fileType);
     void setFlatpakName(const QString &name);
     void setOrigin(const QString &origin);
+    void setDisplayOrigin(const QString &displayOrigin);
     void setPropertyState(PropertyKind kind, PropertyState state);
     void setResourceFile(const QUrl &url);
     void setRuntime(const QString &runtime);
@@ -172,10 +176,6 @@ public:
     QString attentionText() const;
     QString dataLocation() const;
     bool hasDataButUninstalled() const;
-    void setAvailableVersion(const QString &version)
-    {
-        m_availableVersion = version;
-    }
     Q_INVOKABLE QAbstractListModel *permissionsModel();
 
     void setTemporarySource(const QSharedPointer<FlatpakSource> &temp)
@@ -188,6 +188,17 @@ public:
     }
 
     Q_INVOKABLE void clearUserData();
+    Q_INVOKABLE int versionCompare(FlatpakResource *resource) const;
+
+    const AppStream::Component appdata() const
+    {
+        return m_appdata;
+    }
+
+    QString contentRatingText() const override;
+    QString contentRatingDescription() const override;
+    ContentIntensity contentRatingIntensity() const override;
+    uint contentRatingMinimumAge() const override;
 
 Q_SIGNALS:
     void hasDataButUninstalledChanged();
@@ -214,7 +225,8 @@ private:
     AbstractResource::State m_state;
     FlatpakInstallation *const m_installation;
     QString m_origin;
-    QString m_availableVersion;
+    QString m_displayOrigin;
+    mutable QString m_availableVersion;
     FlatpakResource::ResourceType m_type = DesktopApp;
     QSharedPointer<FlatpakSource> m_temp;
     QVector<FlatpakPermission> m_permissions;
